@@ -1,17 +1,20 @@
 package App::ColorThemeUtils;
 
-our $DATE = '2019-02-02'; # DATE
-our $VERSION = '0.002'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-06-07'; # DATE
+our $DIST = 'App-ColorThemeUtils'; # DIST
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
+use Log::ger;
 
 our %SPEC;
 
 $SPEC{list_color_theme_modules} = {
     v => 1.1,
-    summary => 'List color theme modules',
+    summary => 'List ColorTheme modules',
     args => {
         detail => {
             schema => 'bool*',
@@ -20,95 +23,85 @@ $SPEC{list_color_theme_modules} = {
     },
 };
 sub list_color_theme_modules {
-    require PERLANCAR::Module::List;
+    require Module::List::Tiny;
 
     my %args = @_;
 
     my @res;
     my %resmeta;
 
-    my $mods = PERLANCAR::Module::List::list_modules(
+    my $mods = Module::List::Tiny::list_modules(
         "", {list_modules => 1, recurse => 1});
     for my $mod (sort keys %$mods) {
-        next unless $mod =~ /::ColorTheme::/;
+        next unless $mod =~ /(\A|::)ColorTheme::/;
         push @res, $mod;
     }
 
     [200, "OK", \@res, \%resmeta];
 }
 
-$SPEC{list_color_themes} = {
+$SPEC{show_color_theme_swatch} = {
     v => 1.1,
     args => {
         module => {
             schema => 'perl::modname*',
+            req => 1,
             pos => 0,
-            tags => ['category:filtering'],
         },
-        detail => {
-            schema => 'bool*',
-            cmdline_aliases => {l=>{}},
+        module_args => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'module_arg',
+            schema => ['hash*', of=>'str*'],
+        },
+        width => {
+            schema => 'posint*',
+            default => 80,
+            cmdline_aliases => {w=>{}},
         },
     },
 };
-sub list_color_themes {
-    no strict 'refs';
+sub show_color_theme_swatch {
     require Color::ANSI::Util;
-    require PERLANCAR::Module::List;
+    require Color::RGB::Util;
+    require String::Pad;
 
     my %args = @_;
+    my $width = $args{width} // 80;
+    my $mod = $args{module};
+    $mod = "ColorTheme::$mod" unless $mod =~ /(\A|::)ColorTheme::/;
+    (my $modpm = "$mod.pm") =~ s!::!/!g;
+    require $modpm;
 
-    my @mods;
-    if (defined $args{module}) {
-        push @mods, $args{module};
-    } else {
-        my $mods = PERLANCAR::Module::List::list_modules(
-            "", {list_modules => 1, recurse => 1});
-        for my $mod (sort keys %$mods) {
-            next unless $mod =~ /::ColorTheme::/;
-            push @mods, $mod;
-        }
+    my $ctheme = $mod->new(%{ $args{module_args} // {} });
+    my @color_names = $ctheme->get_color_list;
+
+    my $reset = Color::ANSI::Util::ansi_reset();
+    for my $color_name (@color_names) {
+        my $empty_bar = " " x $width;
+        my $color0 = $ctheme->get_color($color_name);
+        my $color_summary = ref $color0 eq 'HASH' && defined($color0->{summary}) ?
+            String::Pad::pad($color0->{summary}, $width, "center", " ", 1) : undef;
+        my $fg_color = ref $color0 eq 'HASH' ? $color0->{fg} : $color0;
+        my $bg_color = ref $color0 eq 'HASH' ? $color0->{bg} : undef;
+        my $color = $fg_color // $bg_color;
+        my $text_bar  = String::Pad::pad(
+            "$color_name (".($fg_color // "-").(defined $bg_color ? " on $bg_color" : "").")",
+            $width, "center", " ", 1);
+        my $bartext_color = Color::RGB::Util::rgb_is_dark($color) ? "ffffff" : "000000";
+        my $bar = join(
+            "",
+            Color::ANSI::Util::ansibg($color), $empty_bar, $reset, "\n",
+            Color::ANSI::Util::ansibg($color), Color::ANSI::Util::ansifg($bartext_color), $text_bar, $reset, "\n",
+            defined $color_summary ? (
+                Color::ANSI::Util::ansibg($color), Color::ANSI::Util::ansifg($bartext_color), $color_summary, $reset, "\n",
+
+            ) : (),
+            Color::ANSI::Util::ansibg($color), $empty_bar, $reset, "\n",
+            $empty_bar, "\n",
+        );
+        print $bar;
     }
-
-    my @res;
-    my %resmeta;
-    for my $mod (@mods) {
-        (my $mod_pm = "$mod.pm") =~ s!::!/!g;
-        require $mod_pm;
-        my $themes = \%{"$mod\::color_themes"};
-        for my $name (sort keys %$themes) {
-            my $colors = $themes->{$name}{colors};
-            my $colorbar = "";
-            for my $colorname (sort keys %$colors) {
-                my $color = $colors->{$colorname};
-                $colorbar .= join(
-                    "",
-                    (length $colorbar ? "" : ""),
-                    ref($color) || !length($color) ? ("   ") :
-                        (
-                            Color::ANSI::Util::ansibg($color),
-                            "   ",
-                            "\e[0m",
-                        ),
-                );
-            }
-            if ($args{detail}) {
-                push @res, {
-                    module => $mod,
-                    name   => $name,
-                    colors => $colorbar,
-                };
-            } else {
-                push @res, "$mod\::$name";
-            }
-        }
-    }
-
-    if ($args{detail}) {
-        $resmeta{'table.fields'} = [qw/module name colors/];
-    }
-
-    [200, "OK", \@res, \%resmeta];
+    [200];
 }
 
 1;
@@ -126,7 +119,7 @@ App::ColorThemeUtils - CLI utilities related to color themes
 
 =head1 VERSION
 
-This document describes version 0.002 of App::ColorThemeUtils (from Perl distribution App-ColorThemeUtils), released on 2019-02-02.
+This document describes version 0.003 of App::ColorThemeUtils (from Perl distribution App-ColorThemeUtils), released on 2020-06-07.
 
 =head1 DESCRIPTION
 
@@ -136,7 +129,7 @@ This distribution contains the following CLI utilities:
 
 =item * L<list-color-theme-modules>
 
-=item * L<list-color-themes>
+=item * L<show-color-theme-swatch>
 
 =back
 
@@ -149,7 +142,7 @@ Usage:
 
  list_color_theme_modules(%args) -> [status, msg, payload, meta]
 
-List color theme modules.
+List ColorTheme modules.
 
 This function is not exported.
 
@@ -159,6 +152,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<detail> => I<bool>
 
+
 =back
 
 Returns an enveloped result (an array).
@@ -173,11 +167,12 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 list_color_themes
+
+=head2 show_color_theme_swatch
 
 Usage:
 
- list_color_themes(%args) -> [status, msg, payload, meta]
+ show_color_theme_swatch(%args) -> [status, msg, payload, meta]
 
 This function is not exported.
 
@@ -185,9 +180,12 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<detail> => I<bool>
+=item * B<module>* => I<perl::modname>
+
+=item * B<module_args> => I<hash>
+
+=item * B<width> => I<posint> (default: 80)
 
-=item * B<module> => I<perl::modname>
 
 =back
 
@@ -228,7 +226,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019, 2018 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2019, 2018 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
